@@ -13,7 +13,7 @@
 #include <sys/syscall.h>
 #include <sys/socket.h>
 #include <sys/types.h>
-#include <stdlib.h> 
+#include <stdlib.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stddef.h>
@@ -34,7 +34,7 @@ struct linux_crypto;
 
 int linux_crypto_init(struct linux_crypto **ctx);
 int linux_crypto_set_key(const uint8_t *key, int keylen,struct linux_crypto *ctx);
-int linux_crypto_decrypt(uint8_t inbuf[], uint8_t outbuf[], int buflen, uint8_t iv[], struct linux_crypto *ctx);
-int linux_crypto_encrypt(uint8_t inbuf[], uint8_t outbuf[], int buflen, uint8_t iv[], struct linux_crypto *ctx);
+int linux_crypto_decrypt(const uint8_t inbuf[], uint8_t outbuf[], int buflen, uint8_t iv[], struct linux_crypto *ctx);
+int linux_crypto_encrypt(const uint8_t inbuf[], uint8_t outbuf[], int buflen, uint8_t iv[], struct linux_crypto *ctx);
 void linux_crypto_free(struct linux_crypto **ctx);
 #endif
